# -*- coding: utf-8 -*-
"""
Combined kwadrants stitching code, Tom Callewaert, Msc.
"""

#import matplotlib.pyplot as plt
import matplotlib.pyplot as plt 
from matplotlib import cm
import numpy as np
import os 
import cv2
import scipy.signal

exec(open("DataAnalysisFunctionsRepository.py").read())

ScatteringStrengthAnalysis = 0
SurfaceFusion = 0
StratigraphyFusion = 0
try:
    DataDirectory = str(r'C:\Users\doreto\surfdrive\ZenodoRepository')   
    os.chdir(DataDirectory)
except:
    print('Please set correct datapaths') 
    
#%% System constants
um_per_pixel = 1.87 # Axial resolution of OCT system
n_glaze = 0.66 # Inverse of the glaze refractive index (1.5**-1)


#%% Example of stitching algorithmn on quadrant basis    
Surface = np.float32(np.load('UpperInterfaceMatrixRawdataKwadrant11.npy'))
AmountRowCubes=41 # Check Experimental Settings
AmountColCubes=41 # Check Experimental Settings
AmountColElements = 3034 # Check Experimental results (amount of slices)
AmountRowElements = 24026 # Check Epxerimental results (# shape[1] of an individual slice)
ChunkIndexRow = np.int64(np.linspace(0, Surface.shape[0],AmountRowCubes+1))
ChunkIndexCol = np.int64(np.linspace(0, Surface.shape[1],AmountColCubes+1))
result = UniversalCubeStitching(Surface,ChunkIndexRow,ChunkIndexCol,clip=[-80,80])

#%% Scattering Strength image
if ScatteringStrengthAnalysis == 1 :
    #Scattering strength arrays
    SS11 = np.float32(np.load('FullResScatteringStrengthKwadrant11.npy')) 
    SS12 = np.float32(np.load('FullResScatteringStrengthKwadrant12.npy'))
    SS21 = np.float32(np.load('FullResScatteringStrengthKwadrant21.npy'))
    SS22 = np.float32(np.load('FullResScatteringStrengthKwadrant22.npy'))
    #% overlay kwadrant 21 & Kwadrant 22    
    LowerPartCombined =np.hstack((SS21[:,:],SS22[:,600:]))
    LowerPartCombined[:,3033] = LowerPartCombined[:,3032]
    #% overlay Kwadrant12 & Kwadrant11
    UpperPartCombined = np.hstack((SS11[:,:],SS12[:,600:]))
    #% combine the figures & Correct for stitching errors
    ScatteringStrengthImage = np.vstack((UpperPartCombined,LowerPartCombined))

    #% plot of the figure  -- Uncorrected Image
    plt.figure()
    plt.imshow(cv2.resize(ScatteringStrengthImage,(3034,3034)), extent=[0,35,0,40], aspect='equal' , cmap='plasma', vmin=0.2, vmax=0.8)
    plt.xlabel('X (cm)', fontsize='15')
    plt.ylabel('Y (cm)', fontsize='15')
    
    #% removal of edge artifacts due to overlapping image stitching
    ChunkIndexRow = np.int64(np.linspace(0, ScatteringStrengthImage.shape[0],41+41+1))
    ChunkIndexCol = np.int64([0,3034,ScatteringStrengthImage.shape[1]])
    ScatteringStrengthImageFused = np.float32(AngularStitch(ScatteringStrengthImage,ChunkIndexRow,ChunkIndexCol,[-100,100]))
    ScatteringStrengthImageFused = ScatteringStrengthImageFused.transpose()        #%%
    ScatteringStrengthImageFused = AngularStitch(ScatteringStrengthImageFused,np.int64([0,3034,ScatteringStrengthImage.shape[1]]),np.int64([0,24026,ScatteringStrengthImage.shape[0]]),[-100,100])
    ScatteringStrengthImageFused = ScatteringStrengthImageFused.transpose()

    #% Plot of the resulting corrected image (downsampling the figure to avoid plotting memory issues)
    plt.figure()
    plt.imshow(cv2.resize(ScatteringStrengthImageFused,(5468,5468)), extent = [0,35,0,40],aspect='equal' , cmap='plasma', vmin=0.2, vmax=0.8)
    plt.xlabel('X (cm)', fontsize='15');  plt.ylabel('Y (cm)', fontsize='15')
    cb = plt.colorbar(); cb.ax.set_ylabel('Scattering strength (arb. units)', rotation=270, fontsize='15', labelpad=20)
   
#%% Surface image 
if SurfaceFusion == 1:
    try:
        Activate_Exception
        SurfaceFused = np.load('SurfaceFullPainting.npy')
    except: #(Computationally expensive)
        SC11 = np.float32(np.load('SurfaceStitchedKwadrant11.npy'))
        SC12 = np.float32(np.load('SurfaceStitchedKwadrant12.npy'))
        SC21 = np.float32(np.load('SurfaceStitchedKwadrant21.npy'))
        SC22 = np.float32(np.load('SurfaceStitchedKwadrant22.npy'))

        AmountRowCubes=41 # Check Experimental Settings
        AmountColCubes=41 # Check Experimental Settings
        AmountColElements = 3034 # Check Experimental results (amount of slices)
        AmountRowElements = 24026 # Check Epxerimental results (# shape[1] of an individual slice)
        ChunkIndexRow = np.int64(np.linspace(0, SC11.shape[0],AmountRowCubes+1))
        ChunkIndexCol = np.int64(np.linspace(0, SC11.shape[1],AmountColCubes+1))

        #% Left & Right Vertical Bar
        VerticalBar1= np.vstack((SC11,SC21))
        VerticalBar1 = UniversalCubeStitching(VerticalBar1, [0,24026,48052], ChunkIndexCol, clip=[-10000,10000])
        #% Right vertical bar
        VerticalBar2 = np.vstack((SC12,SC22))
        VerticalBar2 = UniversalCubeStitching(VerticalBar2, [0,24026,48052], ChunkIndexCol, clip=[-10000,10000])
        #%
        #VerticalBar1 = cv2.resize(VerticalBar1,(3034,3034))
        #VerticalBar2 = cv2.resize(VerticalBar2,(3034,3034))
        SurfaceFused = np.hstack((VerticalBar1,VerticalBar2))
        ChunkIndexRow = np.int64([0,10,48052])
        ChunkIndexCol = np.int64(np.linspace(0, SurfaceFused.shape[1],2+1))
        SurfaceFused = UniversalCubeStitching(SurfaceFused,ChunkIndexRow,ChunkIndexCol,clip=[-10000,10000])
        VerticalBar1 = SurfaceFused[:,0:3034]
        VerticalBar2 = SurfaceFused[:,3034:-1]
        #% Overlapping areas+ recombination
        OC = 600
        CO = np.linspace(0,1,OC);COrev = CO[::-1] 
        Overlay1= VerticalBar1[:,-OC:]*COrev
        Overlay2 =VerticalBar2[:,0:OC]*CO
        Overlay = (Overlay1+Overlay2)
        SurfaceFused = np.hstack((VerticalBar1[:,0:-OC],Overlay,VerticalBar2[:,OC:]))
        #SurfaceFused = cv2.resize(np.hstack((VerticalBar1[:,0:-OC],Overlay,VerticalBar2[:,OC:])),(5468,5468))
        SurfaceFused= scipy.signal.detrend(SurfaceFused,axis=-1)+60
        #np.save('SurfaceFullPainting.npy', SurfaceFused )

    


    #%%
    plt.figure()
    plt.imshow(np.float32(cv2.resize(SurfaceFused*um_per_pixel, (3034,3034))), extent=[0,35,0,40], aspect='equal' , cmap='viridis', vmin = 0 , vmax =255)
    plt.xlabel(' X (cm)', fontsize='15');  plt.ylabel('Y (cm)', fontsize='15')
    cb = plt.colorbar(); cb.ax.set_ylabel('Depth ($\mu$m)', rotation=270, fontsize='15', labelpad=20)
    plt.savefig(str('C:\\Users\\doreto\\surfdrive\\SurfacesForFusion\\' +'SurfaceFullPainting.eps'), format='eps', dpi=300)
#%%
if StratigraphyFusion == 1:
    try:
       Activate_Exception  
       StratigraphyImage = np.load('GlazeStratigraphyFullPainting.npy')
       #StratigraphyImageNoMask = np.load('GlazeStratigraphyFullPaintingNoMask.npy')
       
       print('Datasets loaded')
    except:
        MaskFullPainting = np.load('MaskFullPainting.npy')
        ST11 = np.float32(np.load('FullResGlazeStratigraphyThicknessKwadrant11.npy'))
        ST12 = np.float32(np.load('FullResGlazeStratigraphyThicknessKwadrant12.npy'))
        ST21 = np.float32(np.load('FullResGlazeStratigraphyThicknessKwadrant21.npy'))
        ST22 = np.float32(np.load('FullResGlazeStratigraphyThicknessKwadrant22.npy'))
        
        #% Kwadrant11 &Kwadrant12 overlay
        UpperPartCombined = np.hstack((ST11[:,:],ST12[:,600:]))
        #% overlay kwadrant 21 & Kwadrant 22
        LowerPartCombined =np.hstack((ST21[:,:],ST22[:,600:]))
        #% Fused image
        StratigraphyImage = np.vstack((UpperPartCombined,LowerPartCombined))
        #% Filter the stratigrpahy image
        MaskFullPainting = np.load('MaskFullPainting.npy')
        MaskFullPainting = cv2.resize(MaskFullPainting, (StratigraphyImage.shape[1],StratigraphyImage.shape[0]))
        #StratigraphyImage = cv2.resize(StratigraphyImage, (MaskFullPainting.shape[1],MaskFullPainting.shape[0]))
        StratigraphyImage[MaskFullPainting ==1]=0       
        #np.save('GlazeStratigraphyFullPainting.npy', StratigraphyImage )
    
    plt.figure()
    plt.imshow(cv2.resize(StratigraphyImage*1.87*(1.40),(3034,3034)), aspect='equal' ,cmap='gray',  vmin=20, vmax=70) # extent=[0,35,0,40]
    plt.xlabel('X (cm)', fontsize='15');  plt.ylabel('Y (cm)', fontsize='15')
    cb = plt.colorbar(); cb.ax.set_ylabel('Glaze Thickness ($\mu$m)', rotation=270, fontsize='15', labelpad=20)
    
    #% Console statistics output
    print('Sampling Points = {}'.format(np.size(cv2.resize(((StratigraphyImage)*um_per_pixel*n_glaze),(3034,3034))[:,0:800])))
    print('Average glaze Thickness = {}'.format(np.mean(cv2.resize(((StratigraphyImage)*um_per_pixel*n_glaze),(3034,3034))[:,0:800])))
    print('Standard Deviation= {}'.format(np.std(cv2.resize(((StratigraphyImage)*um_per_pixel*n_glaze), (3034,3034))[:,0:800])))
